/*
Copyright 2014 Samuel Gesche

This file is part of Continuous RSS Reader.

Continuous RSS Reader is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Continuous RSS Reader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ArcEnCiel.  If not, see <http://www.gnu.org/licenses/>.
*/
package continuousrssreader;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import gate.creole.ExecutionException;
import gate.creole.ResourceInstantiationException;
import gate.util.GateException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Timer;

public class MainFrame extends javax.swing.JFrame {
    // The embedder doing the job (here we merely display things)
    private GateEmbedder rssParser;
    
    // The continuous timer
    private int countdown = 0;
    private ActionListener reader = new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent ae) {
            countdown--;
            if(countdown <= 0) {
                read();
            } else {
                status.setText("Waiting " + countdown + " seconds...");
            }
        }
    };
    private Timer readTicker = new Timer(1000, reader);
    
    
    public MainFrame() {
        initComponents();
        try {
            // Constructing our GATE resource
            rssParser = new GateEmbedder();
            
            // First initialisation
            rssParser.init();
            
        } catch (GateException | MalformedURLException ex) {
            err.setText("Error : " + ex.getClass() + " - " + ex.getMessage());
        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        in = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        frequency = new javax.swing.JSpinner();
        jLabel3 = new javax.swing.JLabel();
        read = new javax.swing.JButton();
        jPanel3 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        out = new javax.swing.JEditorPane();
        jPanel4 = new javax.swing.JPanel();
        status = new javax.swing.JLabel();
        jPanel5 = new javax.swing.JPanel();
        err = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        clear = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("RSS Reader");

        jLabel1.setText("Read");
        jPanel1.add(jLabel1);

        in.setColumns(50);
        in.setText("http://rss.nytimes.com/services/xml/rss/nyt/InternationalHome.xml");
        jPanel1.add(in);

        jLabel2.setText("every");
        jPanel1.add(jLabel2);

        frequency.setModel(new javax.swing.SpinnerNumberModel(15, 5, 60, 1));
        jPanel1.add(frequency);

        jLabel3.setText("seconds.");
        jPanel1.add(jLabel3);

        read.setText("Go!");
        read.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                readActionPerformed(evt);
            }
        });
        jPanel1.add(read);

        getContentPane().add(jPanel1, java.awt.BorderLayout.NORTH);

        jPanel3.setLayout(new java.awt.BorderLayout());

        jScrollPane1.setPreferredSize(new java.awt.Dimension(400, 400));

        out.setContentType("text/html"); // NOI18N
        jScrollPane1.setViewportView(out);

        jPanel3.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        jPanel4.add(status);

        jPanel3.add(jPanel4, java.awt.BorderLayout.PAGE_START);

        jPanel5.add(err);

        jPanel3.add(jPanel5, java.awt.BorderLayout.PAGE_END);

        getContentPane().add(jPanel3, java.awt.BorderLayout.CENTER);

        clear.setText("Clear");
        clear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearActionPerformed(evt);
            }
        });
        jPanel2.add(clear);

        getContentPane().add(jPanel2, java.awt.BorderLayout.PAGE_END);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void readActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_readActionPerformed
        clearOut();
        try {
            rssParser.setFeed(new URL(in.getText()));
            read();
        } catch (MalformedURLException ex) {
            err.setText("Error : " + ex.getClass() + " - " + ex.getMessage());
        }
        readTicker.start();
    }//GEN-LAST:event_readActionPerformed

    private void clearActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearActionPerformed
        clearOut();
    }//GEN-LAST:event_clearActionPerformed

    private void clearOut() {
        out.setText("");
        err.setText("");
        status.setText("");
        rssParser.clearEntries();
    }
    
    private void read() {
        countdown = ((Integer)(frequency.getValue()));
        status.setText("Reading...");
        
        try {
            rssParser.readFeed();
            status.setText("Printing...");
            printEntries();
            status.setText("Done.");
        } catch (IOException | ResourceInstantiationException | ExecutionException ex) {
            err.setText("Error : " + ex.getClass() + " - " + ex.getMessage());
        }
    }
    
    private void printEntries() {
        String outText = "";
        for(String entry: rssParser.getEntries()) {
            outText += entry;
        }
        // Some html/css for a better display
        out.setText("<html><head><style>p {text-indent:25px;}</style></head><body><h1>" + rssParser.getEntries().length + " entries</h1>" + outText + "</body></html>");
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton clear;
    private javax.swing.JLabel err;
    private javax.swing.JSpinner frequency;
    private javax.swing.JTextField in;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JEditorPane out;
    private javax.swing.JButton read;
    private javax.swing.JLabel status;
    // End of variables declaration//GEN-END:variables
}
